import http
from typing import Any, Dict, Optional, Sequence, Type

from pydantic import BaseModel, ValidationError, create_model
from pydantic.error_wrappers import ErrorList


class BaseHTTPException(Exception):
    def __init__(
        self,
        status_code: int,
        detail: Optional[str] = None,
        headers: Optional[dict] = None,
    ) -> None:
        if detail is None:
            detail = http.HTTPStatus(status_code).phrase
        self.status_code = status_code
        self.detail = detail
        self.headers = headers

    def __repr__(self) -> str:
        class_name = self.__class__.__name__
        return f"{class_name}(status_code={self.status_code!r}, detail={self.detail!r})"


class WebSocketException(Exception):
    def __init__(self, code: int, reason: Optional[str] = None) -> None:
        self.code = code
        self.reason = reason or ""

    def __repr__(self) -> str:
        cls_name = self.__class__.__name__
        return f"{cls_name}(code={self.code!r}, reason={self.reason!r})"



class HTTPException(BaseHTTPException):
    def __init__(
        self,
        status_code: int,
        detail: Any = None,
        headers: Optional[Dict[str, Any]] = None,
    ) -> None:
        super().__init__(status_code=status_code, detail=detail, headers=headers)


RequestErrorModel: Type[BaseModel] = create_model("Request")
WebSocketErrorModel: Type[BaseModel] = create_model("WebSocket")


class RequestValidationError(ValidationError):
    def __init__(self, errors: Sequence[ErrorList], *, body: Any = None) -> None:
        self.body = body
        super().__init__(errors, RequestErrorModel)


class WebSocketRequestValidationError(ValidationError):
    def __init__(self, errors: Sequence[ErrorList]) -> None:
        super().__init__(errors, WebSocketErrorModel)




import threading
from sqlalchemy.orm import Session


class SessionManager:
    """
    Factory class intended for session control.
    Example of usage:
    def get_session_manager():
        return SessionManager(SessionMaker)
    """
    def __init__(self, session_factory):
        self._current_session = None
        self._current_thread_id = None
        self._session_factory = session_factory

    def get_session(self) -> Session:
        thread_id = threading.current_thread().ident
        if thread_id != self._current_thread_id:
            if self._current_session:
                self._current_session.close()
            self._current_session = self._session_factory()
            self._current_thread_id = thread_id
        return self._current_session




from enum import Enum
from typing import TypeVar, Generic, Optional
from pydantic import BaseModel


class ActionStatus(str, Enum):
    success = "success"
    failure = "failure"


T = TypeVar("T")


class Result(BaseModel, Generic[T]):
    """
    Factory result class.
    Has status, data, error fields.
    Used for general definition of function`s restult.
    Example:
    def send_data_to_remote(data, url):
    	response = post(url, data=data)
    	try:
	    if response.status == 200:
	        return Result.success(data=response.json)
	    else:
	        return Result.failure(error=response.text)
	except Exception as error:
	    return Result.server_error(error=str(error))
    """
    status: ActionStatus
    data: Optional[T]
    error: Optional[str]

    class Config:
        use_enum_values = True

    def is_success(self):
        return self.status == ActionStatus.success

    def is_failure(self):
        return not self.is_success()

    @classmethod
    def success(cls, data: Optional[T] = None):
        return Result(status=ActionStatus.success, data=data, error=None)

    @classmethod
    def failure(cls, error: str):
        return Result(status=ActionStatus.failure, data=None, error=error)

    @classmethod
    def server_error(
        cls,
        error: str = "An error occurred while performing the operation. please try again late or contact support.",
    ):
        return Result(status=ActionStatus.failure, data=None, error=error)





from enum import Enum
from typing import TypeVar, Generic, Optional
from pydantic import BaseModel

from http.exc.http_exc import HTTPException

class ActionStatus(str, Enum):
    success = "success"
    failure = "failure"


T = TypeVar("T")


class HttpResult(BaseModel, Generic[T]):
    """
    Factory either http-result or ws-result class.
    Has status, data, error fields.
    Used for general definition of function`s result.
    You also can import http_status (int, enum) from common.http.status
    and use it status.404_NOT_FOUND.value.
    Example:
    def send_data_to_remote(data, url):
    	response = post(url, data=data)
    	try:
	    if response.status == 200:
	        return Result.success(data=response.json)
	    else:
	        return Result.failure(exc=HTTPException(status=400, detail="Bad request.")
	except Exception as error:
	    return Result.server_error(exc=HTTPException(status=500, detail="Internal server error.")
    """
    status: ActionStatus
    data: Optional[T]
    exc: Exception
    detail: Optional[str]

    class Config:
        use_enum_values = True

    def is_success(self):
        return self.status == ActionStatus.success

    def is_failure(self):
        return not self.is_success()

    @classmethod
    def success(cls, data: Optional[T] = None, detail: Optional[str] = None):
        return HttpResult(status=ActionStatus.success, data=data, detail=detail)

    @classmethod
    def failure(cls, exc: Exception):
        return HttpResult(status=ActionStatus.failure, exc=exc)

    @classmethod
    def server_error(
        cls,
        exc: Exception = HTTPException(status_code=500, detail="Internal server error")
    ):
        return HttpResult(status=ActionStatus.failure, exc=exc)
